/**
 *  Graph
 *
 *  Copyright (C) 2005  Yves Radunz, Volker Grabsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import java.util.*;
import java.lang.*;

public class Graph
{
    private Vector liste=new Vector();
    private double netlength=0;

    /**
     * eine Kante hinzufügen
     */
    public void addedge(int index1, int index2)
    {
	if (!exists(index1, index2)){
	    int[] tmp = new int[2];
	    tmp[0] = index1;
	    tmp[1] = index2;
	    liste.add(tmp);
	}
    }

    /**
     * Anzahl der Kanten
     */
    public int size()
    {
	return liste.size();
    }

    /**
     * existiert die Kante bereits?
     */
    public boolean exists(int index1, int index2)
    {
        if (index1==index2)
            return true;
	for (int i=0;i<size();i++){
	    int[]tmp=edge(i);
	    if ((index1==tmp[0]&&index2==tmp[1])
		||(index1==tmp[1]&&index2==tmp[0]))
		return true;
	}
	return false;
    }

    /**
     * Kante als int[2] ausgeben
     */
    public int[] edge(int nr)
    {
	return (int[])liste.get(nr);
    }

    /**
     * Kante entfernen
     */
    public void removeedge(int nr)
    {
	liste.remove(nr);
    }

    /**
     * alle Kanten löschen
     */
    public void clear()
    {
	liste.removeAllElements();
    }

    /**
     * das Kommunikationsnetz mit der minimalen Laenge bestimmen
     */
    public void optimize(int anzahl, Vector sat, double zeit)
    {
	if (anzahl<=0)
	    return;

        // Laengen der Kanten berechnen
        double[] laenge=new double[liste.size()];
        for (int i=0; i<liste.size(); i++)
	    {
		int[] kante = (int[]) liste.get(i);
		Satellit sat1 = (Satellit)sat.get(kante[0]);
		Satellit sat2 = (Satellit)sat.get(kante[1]);
		laenge[i] = sat1.dist(sat2,zeit);
	    }
	sort(0, laenge.length-1, laenge);

        // Array fuer bereits verbundene Satelliten initialisieren
        boolean[] verbunden=new boolean[anzahl];
        Arrays.fill(verbunden, false);

        // so lange noch nicht verbundene Satelliten hinzugefuegt werden koennen,
        // einen weiteren Satelliten hinzufuegen.
        netlength=0;
        verbunden[0]=true;
        Vector neueliste = new Vector();
        for (int n=next(verbunden); n >= 0; n=next(verbunden))
	    {
		int[] kante = (int[]) liste.get(n);
		netlength += laenge[n];
		verbunden[kante[0]]=true;
		verbunden[kante[1]]=true;
		neueliste.add(kante);
	    }
	if (neueliste.size()!=anzahl-1){
	    neueliste.removeAllElements();
	    netlength=0;
	}
	liste = neueliste;
    }

    /**
     * kleinste Kante zu einem noch nicht verbundenen Satelliten suchen
     */
    private int next(boolean[] verbunden)
    {
        for (int i=0; i<liste.size(); i++)
	    {
		int[] kante = (int[]) liste.get(i);
		if (verbunden[kante[0]] != verbunden[kante[1]])
		    return i;
	    }
        return -1;
    }

    /**
     * Kanten des Graphen mit Quicksort nach der Laenge sortieren
     */
    private void sort(int start, int ende, double[] laenge)
    {
	if (start >= ende)
	    return;

        double m=laenge[(start+ende)/2];
        int i=start;
	int j=ende;
        while (i<=j){
            // Kanten suchen, die groesser als der Trennwert sind
            while(laenge[i]<m)
                i++;

            // Kanten suchen, die kleiner als der Trennwert sind
            while(laenge[j]>m)
                j--;

            // Kanten tauschen
            int[] kante1=(int[])liste.get(i);
            int[] kante2=(int[])liste.get(j);
            liste.set(i, kante2);
            liste.set(j, kante1);

            // Laengen tauschen
            double len=laenge[i];
            laenge[i]=laenge[j];
            laenge[j]=len;

            i++;
	    j--;
        }
        sort(start, j, laenge);
	sort(i, ende, laenge);
    }

    /**
     * Laenge des Graphen
     */
    public double getNetLength()
    {
        return netlength;
        
    }

}