/**
 *  Vektor
 *
 *  Copyright (C) 2005  Yves Radunz, Volker Grabsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

public class Vektor
{
    public double x,y,z;

    public Vektor(double wertx, double werty, double wertz)
    {
        // Startwerte setzen
	set(wertx, werty, wertz);
    }

    public Vektor()
    {
	set(0,0,0);
    }

    /**
     * Werte neu setzen
     */
    public void set(double wertx, double werty, double wertz)
    {
	x=wertx; y=werty; z=wertz;
    }

    /**
     * Werte laden
     */
    public void set(Vektor quelle)
    {
	set(quelle.x, quelle.y, quelle.z);
    }

    public void add(Vektor summand)
    {
        // Vektor zu diesem Vektor addieren
	x+=summand.x; y+=summand.y; z+=summand.z;
    }

    public void mul(Vektor wert)
    {
        // Vektorprodukt mit diesem Vektor
	Vektor tmp=new Vektor(y*wert.z-z*wert.y, z*wert.x-x*wert.z, x*wert.y-y*wert.x);
	set(tmp);
    }

    /**
     * Vektor skalieren
     */
    public void skal(double skalar)
    {
	x*=skalar; y*=skalar; z*=skalar;
    }

    public void neg()
    {
        // Vektor negieren
	skal(-1);
    }

    /**
     * Betrag des Vektors
     */
    public double abs()
    {
	return Math.sqrt(x*x+y*y+z*z);
    }

    /**
     * Vektor ungefähr == Nullvektor?
     */
    public boolean iszero()
    {
        return abs()<.000001;
    }

    /**
     * Skalarprodukt mit Vektor
     */
    public double prod(Vektor wert)
    {
	return wert.x*x+wert.y*y+wert.z*z;
    }
    
    /**
     * Vektor subtrahieren
     */
    public void sub(Vektor wert)
    {
	x-=wert.x; y-=wert.y; z-=wert.z;
    }

    /**
     * Abstand zweier Punkte
     */
    public static double dist(Vektor wert1, Vektor wert2)
    {
	return abs(sub(wert1,wert2));
    }

    /**
     * Vektor in Polarkoordinaten erzeugen
     */
    public static Vektor polar(double radius, double phi, double theta)
    {
	Vektor v = new Vektor(-Math.sin(phi)*Math.cos(theta),
			      Math.sin(theta),
			      -Math.cos(phi)*Math.cos(theta));
	v.skal(radius);
	return v;
    }

    /**
     * Polarkoordinaten und Radius als double[2] aus Vektor erzeugen
     * Index Ausgabe
     * 0     Radius
     * 1     phi
     * 2     theta
     */
    public static double[] polarinv(Vektor v)
    {
	double[] tmp=new double[3];
	tmp[0]=v.abs();
	if (tmp[0]==0) {
	    tmp[1]=0;
	    tmp[2]=0;
	} else{
	    tmp[2]=Math.asin(v.y/tmp[0]);
	    tmp[1]=Math.atan2(-v.x, -v.z);
	}
	return tmp;
    }

    /**
     * Vektorprodukt zweier Vektoren
     */
    public static Vektor mul(Vektor wert1, Vektor wert2)
    {
	Vektor tmp=new Vektor(wert1.x, wert1.y, wert1.z);
	tmp.mul(wert2); return tmp;
    }

    /**
     * Vektoren addieren
     */
    public static Vektor add(Vektor wert1, Vektor wert2)
    {
	Vektor tmp=new Vektor(wert1.x, wert1.y, wert1.z);
	tmp.add(wert2); return tmp;
    }

    /**
     * Vektor skalieren
     */
    public static Vektor skal(double skalar, Vektor wert)
    {
	Vektor tmp=new Vektor(wert.x, wert.y, wert.z);
	tmp.skal(skalar); return tmp;
    }

    /**
     * Vektor negieren
     */
    public static Vektor neg(Vektor wert)
    {
        Vektor tmp=new Vektor(-wert.x, -wert.y, -wert.z);
	return tmp;
    }

    /**
     * Betrag des Vektors
     */
    public static double abs(Vektor wert)
    {
	return wert.abs();
    }

    /**
     * Vektor ungefähr Null?
     */
    public static boolean iszero(Vektor wert)
    {
	return wert.iszero();
    }

    /**
     * Skalarprodukt zweier Vektoren
     */
    public static double prod(Vektor wert1, Vektor wert2)
    {
	Vektor tmp=new Vektor(wert1.x, wert1.y, wert1.z);
	return tmp.prod(wert2);
    }

    /**
     * Differenz zweier Vektoren
     */
    public static Vektor sub(Vektor wert1, Vektor wert2)
    {
	Vektor tmp=new Vektor(wert1.x, wert1.y, wert1.z);
	tmp.sub(wert2);
	return tmp;
    }

    /**
     * Abstand der Geraden vom Koordinatenursprung
     */
    public static double distGerade(Vektor stuetz, Vektor richt)
    {
	double tmp=Vektor.prod(richt,stuetz)/richt.abs();
	return Math.sqrt(Vektor.prod(stuetz, stuetz)-tmp*tmp);

    }

    /**
     * DEBUG: Werte ausgeben
     */
    public void print()
    {
	System.out.print("("+x+", "+y+", "+z+")");
    }
}
